<?php
/**
 * Ticketsoft API Requests
 * 
 * This file is included in inc.lib.html.php. It serves
 * as the ticketsoft API connection layer.
 * 
 * @package wwm_lib_php
 * @version 0.1
 * @author Rob Fonseca <rob@westworldmedia.com>
 * @copyright West World Media ©2013
 */
/*********************
Ticketsoft API Requests
Written by Rob Fonseca
West World Media ©2013
**********************/

//Send data via POST to URL and get back results
function ticketsoft_SendRequest($request) {
	
	//TEST URL
	$url = "http://192.168.30.5:5006/catch.asp";
	
	// LIVE URL
	//$url = "http://192.168.30.5:5006/catch.asp";
	
	// Send the request to Ticketsoft
	$ch = curl_init($url);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10); // By default curl will wait indefinitely for "Connection timed out"
	curl_setopt($ch, CURLOPT_POSTFIELDS, ($request));
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	
	//get the response from Ticketsoft
	$response = curl_exec($ch);
	
	//close the connection
	curl_close($ch);
	
	//return the response
	return $response;
}

/*
* Build a command to send to ticketsoft (the command, the extra info needed for this command, '')
* Each command is a set of Name/Value pairs separated by a space and
* delimited by a CRLF (Codes Ascii 13 and Ascii 10).  Additionally,
* commands must end with a period (â€œ.â€) on a new line after the command.
* Although there is further detail in the document, a quick example of the
* proper command structure looks like this:

COMMAND CARD_BALANCE
USER_ID ADMN
USER_PW gl123
CARD_NUM 80001234567890
STORE_ID 27
AUTH_ID 345
TERMINAL_ID 11

* @param $command (constructed by other functions like CardRegister, etc..)
* @param $pairs (constructed by other functions like CardRegister, etc..)
* @param $transpairs (constructed by other functions like CardRegister, etc..)
*/
function ticketsoft_Command($command, $pairs, $transpairs) {
		
	//clear some variables
	$output = "";
	
	//Global Credentials
	$storeid = "0";
	$authid = "0";
	$terminalid = "0";
	$vendor = "WWM";
	$transdate = date('m/d/Y');
	
	
	///TEST server credentials
	//$username = "epctst";
	//$password = "wwmtst517";
	
	//LIVE Server credentials
	$username = "parwwm";
	$password = "parwwm3d";
	
	$output =
		"COMMAND $command\r\n" .
		"USER_ID $username\r\n".
		"USER_PW $password\r\n";
	
	if ($pairs != "")
		$output .= $pairs;
		
	$output .=
		"STORE_ID $storeid\r\n".
		"AUTH_ID $authid\r\n".
		"TERMINAL_ID $terminalid\r\n";
	
	if ($transpairs != "")
		$output .= $transpairs;

	$output .= "EXT_VENDOR_CODE $vendor\r\n.";

	return $output;
}

/*
* Registers a loyalty/Gift Card card
*
* @param $cardnumber
* @param $customerid (ie: 12345)
* @param $customeremail (ie: foo@bar.com)
*/
function ticketsoft_CardRegister($card_number, $user_id, $first_name, $last_name, $email_address, $address, $city, $state, $zip, $phone) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command
	$command = "CARD_REGISTER";
	$pairs  = "CARD_DESC " . substr("$first_name $last_name loyalty card",0,50) . "\r\n";
	$pairs  .= "CARD_NUM $card_number\r\n";
	$pairs  .= "SITE_USER_ID $user_id\r\n";
	$pairs  .= "SITE_EMAIL $email_address\r\n";
	$pairs  .= "FIRST_NAME " . substr($first_name,0,30) . "\r\n";
	$pairs  .= "LAST_NAME " . substr($last_name,0,30) . "\r\n";
	$pairs  .= "ADDRESS1 " . substr($address,0,30) . "\r\n";
	$pairs  .= "CITY " . substr($city,0,30) . "\r\n";
	$pairs  .= "STATE $state\r\n";
	$pairs  .= "ZIP_CODE " . substr($zip,0,10) . "\r\n";
	$pairs  .= "PHONE_HOME $phone\r\n";
	$pairs  .= "PHONE_CELL $phone\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}

/*
* Registers a loyalty/Gift Card card
*
* @param $cardnumber
* @param $customerid (ie: 12345)
* @param $customeremail (ie: foo@bar.com)
*/
function ticketsoft_UpdateProfile($card_number, $user_id, $first_name, $last_name, $email_address, $address, $city, $state, $zip, $phone) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
		
	//set some variables and build the full command
	$command = "PROFILE_UPDATE";
	$pairs  = "CARD_DESC " . substr("$first_name $last_name loyalty card",0,50) . "\r\n";
	$pairs  .= "CARD_NUM $card_number\r\n";
	$pairs  .= "SITE_USER_ID $user_id\r\n";
	$pairs  .= "SITE_EMAIL $email_address\r\n";
	$pairs  .= "FIRST_NAME " . substr($first_name,0,30) . "\r\n";
	$pairs  .= "LAST_NAME " . substr($last_name,0,30) . "\r\n";
	$pairs  .= "ADDRESS1 " . substr($address,0,30) . "\r\n";
	$pairs  .= "CITY " . substr($city,0,30) . "\r\n";
	$pairs  .= "STATE $state\r\n";
	$pairs  .= "ZIP_CODE " . substr($zip,0,10) . "\r\n";
	$pairs  .= "PHONE_HOME $phone\r\n";
	$pairs  .= "PHONE_CELL $phone\r\n";
	$request = ticketsoft_Command($command, $pairs, "");

	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}


/*
* Gets the point/cummulative point balance for a loyalty/gift card
*
* @param $cardnumber
*/
function ticketsoft_CardBalance($cardnumber) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command
	$command = "CARD_BALANCE";
	$pairs  = "CARD_NUM $cardnumber\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}

/*
* Merges a loyalty card balance with a new card and returns the servers response
*
* @param $oldcardnumber
* @param $newcardnumber
*/
function ticketsoft_CardMerge($oldcardnumber, $newcardnumber) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command<br />
	$transdate = date('m/d/Y');
	
	$command = "CARD_MERGE";
	$pairs  = "SRC_CARD_NUMS $oldcardnumber\r\n";
	$pairs  .= "CARD_NUM $newcardnumber\r\n";
	$pairs  .= "STORE_TRANS_DATE $transdate\r\n";
	$pairs  .= "STORE_TRANS_TYPE 1\r\n";
	$pairs  .= "STORE_TRANS_NUM 0\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}

/*
* Adds points to a loyalty/gift card
*
* @param $card_number
* @param $points (ie: 5)
*/
function ticketsoft_PointAdd($cardnumber, $points) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command<br />
	$transdate = date('m/d/Y');
	
	$command = "POINT_ADD";
	$pairs  = "CARD_NUM $cardnumber\r\n";
	$pairs  .= "POINT_VALUE $points\r\n";
	$pairs  .= "STORE_TRANS_DATE $transdate\r\n";
	$pairs  .= "STORE_TRANS_TYPE 1\r\n";
	$pairs  .= "STORE_TRANS_NUM 0\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}

/*
* Deducts points from a loyalty/gift card
*
* @param $card_number
* @param $points (ie: 5)
*/
function ticketsoft_PointDeduct($cardnumber, $points) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command<br />
	$transdate = date('m/d/Y');
	
	$command = "POINT_DEDUCT";
	$pairs  = "CARD_NUM $cardnumber\r\n";
	$pairs  .= "POINT_VALUE $points\r\n";
	$pairs  .= "STORE_TRANS_DATE $transdate\r\n";
	$pairs  .= "STORE_TRANS_TYPE 1\r\n";
	$pairs  .= "STORE_TRANS_NUM 0\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}

/*
* Adds a coupon to a loyalty card 
*
* @param $card_number
* @param array $reward
*/
function ticketsoft_CouponAdd($card_number, $reward) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command<br />
	$transdate = date('m/d/Y');
	
	$command = "COUPON_ADD";
	$pairs  = "PROMO_CODE LOYALTY\r\n";
	$pairs  .= "CARD_NUM $card_number\r\n";
	$pairs  .= "ITEM_TYPE " . $reward["type"] . "\r\n";
	$pairs  .= "ITEM_ID " . $reward["item_id"] . "\r\n";
	$pairs  .= "DISC_TYPE " . $reward["disc_type"] . "\r\n";
	$pairs  .= "DISC_AMT " . $reward["disc_amt"] . "\r\n";
	$pairs  .= "DISC_QTY " . $reward["qty"] . "\r\n";
	$pairs  .= "HEADER_TEXT " . $reward["header"] . "\r\n";
	$pairs  .= "DETAIL_TEXT " . $reward["detail"] . "\r\n";
	$pairs  .= "VALID_DATE " . $reward["valid_date"] . "\r\n";
	$pairs  .= "EXP_DATE " . $reward["exp_date"] . "\r\n";
	$pairs  .= "STORE_TRANS_DATE $transdate\r\n";
	$pairs  .= "STORE_TRANS_TYPE 1\r\n";
	$pairs  .= "STORE_TRANS_NUM 0\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}

/*
* Checks coupons/rewards that are assigned to a loyalty card
*
* @param $card_number
* @param $amount (ie: 5.00)
*/
function ticketsoft_CouponList($card_number) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command<br />
	$transdate = date('m/d/Y');
	
	$command = "COUPON_LIST";
	$pairs  = "PROMO_CODE LOYALTY\r\n";
	$pairs  .= "CARD_NUM $card_number\r\n";
	$pairs  .= "LIST_MODE REPRINT\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $response;
}


/*
* Adds a virtual balance to a loyalty card. Used as cash at the POS
*
* @param $card_number
* @param $amount (ie: 5.00)
*/
function ticketsoft_VirtualBalanceAdd($card_number, $amount) {
	
	//clear some variables
	$command = "";
	$request = "";
	$response = "";
	$output = "";
	$pairs = "";
	
	//set some variables and build the full command<br />
	$transdate = date('m/d/Y');
	
	$command = "VBALANCE_ADD";
	$pairs  .= "CARD_NUM $card_number\r\n";
	$pairs  .= "AMOUNT $amount\r\n";
	$pairs  .= "STORE_TRANS_DATE $transdate\r\n";
	$pairs  .= "STORE_TRANS_TYPE 1\r\n";
	$pairs  .= "STORE_TRANS_NUM 0\r\n";
	$request = ticketsoft_Command($command, $pairs, "");
	
	//send the request to Ticketsoft
	$response = ticketsoft_SendRequest($request);
	
	//return the request to and response from Ticketsoft
	return $request.','.$response;
}

/*
* Gets the value of a particular field by passing in the servers response string and a field name 
*
* @param $buffer (the full response string from ticketsoft)
* @param $parameter (the field you wish to extract from that response)
*/
function ticketsoft_GetReturn($buffer, $parameter)
{
	$pos1 = "";
	$pos2 = "";
	$output = "";
	
	//Find the first CRLF (and there has to be one)
	$pos1 = strpos($buffer, "\r\n");
	if ($pos1 == 0)
		die();
	
	//if this is the first field in the args or response, handle it differently
	if ($parameter == 'COMMAND' || $parameter == 'TERMINATION_STATUS') {	
		$pos1 = strpos($buffer,' ') + 1;
		$pos2 = strpos($buffer, "\r\n", $pos1);
		$output = substr($buffer,$pos1,($pos2 - $pos1));
				
		if ($parameter == "TERMINATION_STATUS") {
			if (trim($output) == "SUCCESS") {
				$output = "1";
			} else {
				$output = "0";
			}
		}
		
		return $output;
	} else {
		//Find start of param name after first CRLF
		$pos1 = strpos($buffer, $parameter, $pos1);
		if ($pos1 > 0) {
			//Find the CR after the param
			$pos2 = strpos($buffer, "\r\n", $pos1);
			if ($pos2 > 0) {
				$pos1 = ($pos1 + strlen($parameter) + 1);
				return substr($buffer, $pos1, ($pos2 - $pos1));
			}
		}
	}
}

/*
* Gets all valid coupons for a loyalty card
*
* @param $response (the full response string from ticketsoft for COUPON_LIST)
*
* return two dimensional array $coupons
*/
function ticketsoft_GetCoupons($card_number) {
	
	//setup array
	$coupons = array();
	
	//get all valid coupons
	$response = ticketsoft_CouponList($card_number);
	
	//find out how many coupons we have
	$num_coupons = ticketsoft_GetReturn($response, 'COUPON_COUNT');
	
	//loop through all the coupons from 1 to $num_coupons
	for ($i = 1; $i <= intval($num_coupons); $i++) {
		
		//add 0 infront of $i if less than 10
		if ($i < 10) {
			$c_num = "0" . $i;
		} else {
			$c_num = $i;
		}
		
		//get the detail string for this coupon
		$c_detail = ticketsoft_GetReturn($response, "COUPON_$c_num");
		
		//split the coupon detail string into an array
		$c_fields = explode('~',$c_detail);
		
		//fill in the $coupons array with this coupons details
		$coupons["$i"]["coupon_number"] = $c_fields[2];
		$coupons["$i"]["issue_date"] = $c_fields[3];
		$coupons["$i"]["issue_date"] = $c_fields[3];
		$coupons["$i"]["exp_date"] = $c_fields[4];
		$coupons["$i"]["item"] = $c_fields[6];
		$coupons["$i"]["item_id"] = $c_fields[7];
		$coupons["$i"]["header"] = $c_fields[11];
		$coupons["$i"]["detail"] = $c_fields[12];
		$coupons["$i"]["sponsor_text"] = $c_fields[13];
	}
	
	//return the array
	return $coupons;
}

/**
 * adds a row to the Ticketsoft log table
 * 
 * @param $db
 * @param $user_id
 * @param $card_number
 * @param $response
 * @param $reason
 * 
 */
function ticketsoft_log($db,$user_id,$authorized_id,$card_number,$args,$response,$reason, $point_value) {
		
	//get the command
	$command = ticketsoft_GetReturn($args, 'COMMAND');
	$success = ticketsoft_GetReturn($response, 'TERMINATION_STATUS');
	
	//log this API transaction
	try {
		$sql = "INSERT INTO Loyalty..Epic_TicketsoftLog 
				(user_id,card_number,command,point_value,args,response,success,reason,authorized_id,created)
				VALUES (:user_id,:card_number,:command,:point_value,:args,:response,:success,:reason,:authorized_id,:created)";
		$q = $db->prepare($sql);
		$q->execute(array(':user_id' => $user_id,':card_number' => $card_number,':command' => $command,
						  ':point_value' => $point_value,':args' => $args,':response' => $response,
						  ':success' => $success,':reason' => $reason,':authorized_id' => $authorized_id, ':created' => date('Y-m-d H:i:s')));
	} catch (PDOException $e) {
		handle_exception($e);
	}
}

/**
 * splits a ticketsoft API response into an array
 * of argument and response from an API request
 * 
 * @param $api_response
 * 
 * return two dimensional array $ts
 */
function ticketsoft_GetCommandArgs($api_response) {
	
	//intialize return array
	$ts = array();
	
	//split the response into command and response
	$ts_return = explode(",", $api_response);
	$ts['args'] = $ts_return[0];
	$ts['response'] = $ts_return[1];
	
	//return the array
	return $ts;
}
?>