<?php
/**
 * User input sanitization, validation, and gorilla proofing.
 * 
 * This file contains functionality for sanitizing and validating
 * user input. 
 * 
 * @package wwm_lib_php
 * @version 0.2
 * @author Daniel Roestorf <danielroestorf@gmail.com>
 * @copyright West World Media ©2013
 */

	
	/**
	 * Sanitize a string.
	 * 
	 * @param $str
	 * @return String
	 */
	function sanitize_string($str) {
		return filter_var(trim($str), FILTER_SANITIZE_STRING);
	}
	
	/**
	 * Sanitize an email address.
	 * 
	 * @param $str
	 * @return String
	 */
	function sanitize_email($str) {
		return filter_var(trim($str), FILTER_SANITIZE_EMAIL);
	}

	/**
	 * Check if an email address is valid.
	 * 
	 * @param $str
	 * @return boolean
	 */
	function validate_email($str) {
		$str = trim($str);
		if (filter_var($str, FILTER_VALIDATE_EMAIL) && strpos($str, '@') !== false) {
			return true;
		}
		return false;
	}
	
	/**
	 * Checks if a string is valid by checking its length
	 * against $len.
	 * 
	 * @param $str
	 * @param $len the minimum length
	 * @return boolean
	 */
	function validate_string($str) {
		$str = trim($str);
		if (strlen($str) > 1) 
			return true;
		return false; 
	}
	
	/**
	 * Loose password validation, simply checks that it is 
	 * at least 8 characters long.
	 * 
	 * @param $password
	 * @return Boolean
	 */
	function validate_password($password) {
		if (strlen(trim($password)) > 7) {
			return true;
		}
		return false;
	}
	
	/**
	 * Checks if a card number is numeric and that its length 
	 * is 16 characters.
	 * 
	 * @param $num
	 * @return Boolean
	 */
	function validate_card_number($num) {
		if (is_numeric($num) && strlen($num) === 16) {
			return true;
		}
		return false;
	}
	
	/**
	 * Validates zip codes by checking if they're either 5 or 9
	 * digits after striping out any non-numeric numbers.
	 * 
	 * @param $zip
	 * @return Boolean
	 */
	function validate_zip_code($zip) {
		$zip = preg_replace('/\D/', '', $zip);
		if (is_numeric($zip) && strlen($zip) === 5 || strlen($zip) === 9) {
			return true;
		}
		return false;
	}

?>